@@ -30,10 +30,11 @@ from safe_cli.api.transaction_service_api import TransactionServiceApi
 from safe_cli.ethereum_hd_wallet import get_account_from_words
 from safe_cli.safe_addresses import (
     LAST_DEFAULT_CALLBACK_HANDLER,
+    LAST_MULTISEND_CALL_ONLY_CONTRACT,
     LAST_MULTISEND_CONTRACT,
     LAST_SAFE_CONTRACT,
 )
-from safe_cli.utils import yes_or_no_question
+from safe_cli.utils import get_erc_20_list, yes_or_no_question
 
 try:
     from functools import cached_property
@@ -341,7 +342,7 @@ class SafeOperator:
             sender_account = sender_account[0]
             transaction_to_send = self.safe_contract.functions.approveHash(
                 hash_to_approve
-            ).buildTransaction(
+            ).build_transaction(
                 {
                     "from": sender_account.address,
                     "nonce": self.ethereum_client.get_nonce_for_account(
@@ -387,7 +388,7 @@ class SafeOperator:
             # TODO Allow to set threshold
             transaction = self.safe_contract.functions.addOwnerWithThreshold(
                 new_owner, threshold
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.owners = self.safe.retrieve_owners()
                 self.safe_cli_info.threshold = threshold
@@ -409,7 +410,7 @@ class SafeOperator:
             )
             transaction = self.safe_contract.functions.removeOwner(
                 prev_owner, owner_to_remove, threshold
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.owners = self.safe.retrieve_owners()
                 self.safe_cli_info.threshold = threshold
@@ -440,7 +441,7 @@ class SafeOperator:
         transaction = (
             get_erc20_contract(self.ethereum_client.w3, token_address)
             .functions.transfer(to, amount)
-            .buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            .build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
         )
         return self.send_custom(
             token_address, 0, HexBytes(transaction["data"]), **kwargs
@@ -450,7 +451,7 @@ class SafeOperator:
         transaction = (
             get_erc721_contract(self.ethereum_client.w3, token_address)
             .functions.transferFrom(self.address, to, token_id)
-            .buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            .build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
         )
         return self.send_custom(token_address, 0, transaction["data"], **kwargs)
 
@@ -471,7 +472,7 @@ class SafeOperator:
         else:
             transaction = self.safe_contract.functions.setFallbackHandler(
                 new_fallback_handler
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.fallback_handler = new_fallback_handler
                 self.safe_cli_info.version = self.safe.retrieve_version()
@@ -487,9 +488,9 @@ class SafeOperator:
         elif guard != NULL_ADDRESS and not self.ethereum_client.is_contract(guard):
             raise InvalidGuardException(f"{guard} address is not a contract")
         else:
-            transaction = self.safe_contract.functions.setGuard(guard).buildTransaction(
-                {"from": self.address, "gas": 0, "gasPrice": 0}
-            )
+            transaction = self.safe_contract.functions.setGuard(
+                guard
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.guard = guard
                 self.safe_cli_info.version = self.safe.retrieve_version()
@@ -507,7 +508,7 @@ class SafeOperator:
 
             transaction = self.safe_contract_1_1_0.functions.changeMasterCopy(
                 new_master_copy
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.master_copy = new_master_copy
                 self.safe_cli_info.version = self.safe.retrieve_version()
@@ -536,10 +537,10 @@ class SafeOperator:
             for data in (
                 self.safe_contract_1_1_0.functions.changeMasterCopy(
                     LAST_SAFE_CONTRACT
-                ).buildTransaction(tx_params)["data"],
+                ).build_transaction(tx_params)["data"],
                 self.safe_contract_1_1_0.functions.setFallbackHandler(
                     LAST_DEFAULT_CALLBACK_HANDLER
-                ).buildTransaction(tx_params)["data"],
+                ).build_transaction(tx_params)["data"],
             )
         ]
 
@@ -567,7 +568,7 @@ class SafeOperator:
         else:
             transaction = self.safe_contract.functions.changeThreshold(
                 threshold
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
 
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.threshold = threshold
@@ -580,7 +581,7 @@ class SafeOperator:
         else:
             transaction = self.safe_contract.functions.enableModule(
                 module_address
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.modules = self.safe.retrieve_modules()
 
@@ -597,7 +598,7 @@ class SafeOperator:
                 previous_address = self.safe_cli_info.modules[pos - 1]
             transaction = self.safe_contract.functions.disableModule(
                 previous_address, module_address
-            ).buildTransaction({"from": self.address, "gas": 0, "gasPrice": 0})
+            ).build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
             if self.execute_safe_internal_transaction(transaction["data"]):
                 self.safe_cli_info.modules = self.safe.retrieve_modules()
 
@@ -754,6 +755,68 @@ class SafeOperator:
             )
         return False
 
+    # Batch_transactions multisend
+    def batch_safe_txs(self, safe_nonce: int, safe_txs: [SafeTx]) -> SafeTx:
+        """
+        Submit signatures to the tx service. It's recommended to be on Safe v1.3.0 to prevent issues
+        with `safeTxGas` and gas estimation.
+
+        :return:
+        """
+        if not self.ethereum_client.is_contract(LAST_MULTISEND_CALL_ONLY_CONTRACT):
+            print_formatted_text(
+                HTML(
+                    f"<ansired>Multisend call only contract {LAST_MULTISEND_CALL_ONLY_CONTRACT} "
+                    f"is not deployed on this network and it's required for batching txs</ansired>"
+                )
+            )
+
+        multisend_txs = []
+        for safe_tx in safe_txs:
+            # Check if call is already a Multisend call
+            inner_txs = MultiSend.from_transaction_data(safe_tx.data)
+            if inner_txs:
+                multisend_txs.extend(inner_txs)
+            else:
+                multisend_txs.append(
+                    MultiSendTx(
+                        MultiSendOperation.CALL, safe_tx.to, safe_tx.value, safe_tx.data
+                    )
+                )
+
+        if len(multisend_txs) > 1:
+            multisend = MultiSend(
+                LAST_MULTISEND_CALL_ONLY_CONTRACT, self.ethereum_client
+            )
+            safe_tx = SafeTx(
+                self.ethereum_client,
+                self.address,
+                LAST_MULTISEND_CALL_ONLY_CONTRACT,
+                0,
+                multisend.build_tx_data(multisend_txs),
+                SafeOperation.DELEGATE_CALL.value,
+                0,
+                0,
+                0,
+                None,
+                None,
+                safe_nonce=safe_nonce,
+            )
+        else:
+            safe_tx.safe_tx_gas = 0
+            safe_tx.base_gas = 0
+            safe_tx.gas_price = 0
+            safe_tx.signatures = b""
+            safe_tx.safe_nonce = safe_nonce  # Resend single transaction
+        safe_tx = self.sign_transaction(safe_tx)
+        if not safe_tx.signatures:
+            print_formatted_text(
+                HTML("<ansired>At least one owner must be loaded</ansired>")
+            )
+            return None
+        else:
+            return safe_tx
+
     # TODO Set sender so we can save gas in that signature
     def sign_transaction(self, safe_tx: SafeTx) -> SafeTx:
         permitted_signers = self.get_permitted_signers()
@@ -811,6 +874,55 @@ class SafeOperator:
     def get_permitted_signers(self) -> Set[str]:
         return set(self.safe_cli_info.owners)
 
+    def drain(self, to: str):
+        # Getting all events related with ERC20 transfers
+        last = self.ethereum_client.get_block("latest")["number"]
+        token_addresses = get_erc_20_list(self.ethereum_client, self.address, 1, last)
+        safe_txs = []
+        for token_address in token_addresses:
+            balance = self.ethereum_client.erc20.get_balance(
+                self.address, token_address
+            )
+            if balance > 0:
+                transaction = (
+                    get_erc20_contract(self.ethereum_client.w3, token_address)
+                    .functions.transfer(to, balance)
+                    .build_transaction({"from": self.address, "gas": 0, "gasPrice": 0})
+                )
+
+                safe_tx = self.prepare_safe_transaction(
+                    token_address,
+                    0,
+                    HexBytes(transaction["data"]),
+                    SafeOperation.CALL,
+                    safe_nonce=None,
+                )
+                safe_txs.append(safe_tx)
+        # Getting ethereum balance
+        balance_eth = self.ethereum_client.get_balance(self.address)
+        if balance_eth > 0:
+            safe_tx = self.prepare_safe_transaction(
+                to,
+                balance_eth,
+                b"",
+                SafeOperation.CALL,
+                safe_nonce=None,
+            )
+            safe_txs.append(safe_tx)
+        if len(safe_txs) > 0:
+            multisend_tx = self.batch_safe_txs(self.get_nonce(), safe_txs)
+            if multisend_tx is not None:
+                if self.execute_safe_transaction(multisend_tx):
+                    print_formatted_text(
+                        HTML(
+                            "<ansigreen>Transaction to drain account correctly executed</ansigreen>"
+                        )
+                    )
+        else:
+            print_formatted_text(
+                HTML("<ansigreen>Safe account is currently empty</ansigreen>")
+            )
+
     def process_command(self, first_command: str, rest_command: List[str]) -> bool:
         if first_command == "help":
             print_formatted_text("I still cannot help you")
