@@ -8,6 +8,7 @@ from web3 import Web3
 
 from gnosis.eth import EthereumClient
 from gnosis.safe import Safe
+from gnosis.safe.multi_send import MultiSend
 
 from safe_cli.operators.safe_operator import (
     AccountNotLoadedException,
@@ -27,6 +28,8 @@ from safe_cli.operators.safe_operator import (
     SameMasterCopyException,
     SenderRequiredException,
 )
+from safe_cli.utils import get_erc_20_list
+from tests.utils import generate_transfers_erc20
 
 from .safe_cli_test_case_mixin import SafeCliTestCaseMixin
 
@@ -219,6 +222,56 @@ class SafeCliTestCase(SafeCliTestCaseMixin, unittest.TestCase):
         self.assertTrue(safe_operator.send_ether(random_address, value))
         self.assertEqual(self.ethereum_client.get_balance(random_address), value)
 
+    def test_drain(self):
+        safe_operator = self.setup_operator()
+        account = Account.create()
+        value = self.w3.toWei(10.5, "ether")
+        with self.assertRaises(NotEnoughEtherToSend):
+            safe_operator.send_ether(account.address, value)
+        self.ethereum_client.send_eth_to(
+            self.ethereum_test_account.key,
+            safe_operator.address,
+            self.w3.eth.gas_price,
+            value,
+            gas=50000,
+        )
+        safe_operator.send_ether(account.address, value - 5)
+        # Deploying 3 contracts and sending 3 transactions of 1 ERC20 per contract
+        num_transactions = 3
+        num_contracts_erc20 = 3
+        generate_transfers_erc20(
+            self.w3, safe_operator, account, num_contracts_erc20, num_transactions
+        )
+        # Deploying multisend contract on ganache
+        _, _, contract_address = MultiSend.deploy_contract(
+            self.ethereum_client, account
+        )
+        # Getting events filtered by Transfer
+        last = safe_operator.ethereum_client.get_block("latest")["number"]
+        token_address = get_erc_20_list(
+            safe_operator.ethereum_client, safe_operator.address, 1, last
+        )
+        self.assertEqual(len(token_address), num_contracts_erc20)
+        for token in token_address:
+            result = self.ethereum_client.erc20.get_balance(
+                safe_operator.address, token
+            )
+            self.assertEqual(result, num_transactions)
+        # Draining the account to a new account
+        with mock.patch(
+            "safe_cli.operators.safe_operator.LAST_MULTISEND_CALL_ONLY_CONTRACT",
+            contract_address,
+        ):
+            safe_operator.drain(account.address)
+        # Checking that the account is empty
+        for token in token_address:
+            self.assertEqual(
+                self.ethereum_client.erc20.get_balance(safe_operator.address, token), 0
+            )
+        self.assertEqual(
+            safe_operator.ethereum_client.get_balance(safe_operator.address), 0
+        )
+
 
 if __name__ == "__main__":
     unittest.main()
